use gloo_timers::callback::Interval;
use instant::Instant;
use yew::prelude::*;

enum Msg {
    Start,
    Stop,
    UpdateDuration,
}

struct LockWatch {
    duration: Option<Instant>,
    duration_handle: Option<Interval>,
}

impl LockWatch {
    fn get_duration(&self) -> String {
        self.duration.map_or_else(
            || "00:00:00.000".to_string(),
            |instant| {
                let elapsed = instant.elapsed();

                format!(
                    "{:02}:{:02}:{:02}.{:03}",
                    elapsed.as_secs() / 60 / 60,
                    (elapsed.as_secs() / 60) % 60,
                    elapsed.as_secs() % 60,
                    elapsed.as_millis() % 1_000
                )
            },
        )
    }
}

impl Component for LockWatch {
    type Message = Msg;
    type Properties = ();

    fn create(_ctx: &Context<Self>) -> Self {
        Self {
            duration: None,
            duration_handle: None,
        }
    }

    fn update(&mut self, ctx: &Context<Self>, msg: Self::Message) -> bool {
        match msg {
            Msg::Start => {
                self.duration = Some(Instant::now());
                self.duration_handle = {
                    let link = ctx.link().clone();
                    Some(Interval::new(1, move || {
                        link.send_message(Msg::UpdateDuration);
                    }))
                };
            }
            Msg::Stop => self.duration_handle = None,
            Msg::UpdateDuration => {}
        }

        true
    }

    fn view(&self, ctx: &Context<Self>) -> Html {
        html! {
            <section class="section">
                <div class="container">
                    <h1 class="title">{ "LockWatch" }</h1>
                    <div class="block is-size-1">
                        { &self.get_duration() }
                    </div>
                    <div class="block">
                        if self.duration_handle.is_some() {
                            <button class="button is-danger" onclick={ctx.link().callback(|_| Msg::Stop)}>{ "Stop" }</button>
                        } else {
                            if self.duration.is_some() {
                                <button class="button" disabled=true>{ "Start" }</button>
                            } else {
                                <button class="button is-primary" onclick={ctx.link().callback(|_| Msg::Start)}>{ "Start" }</button>
                            }
                        }
                    </div>
                </div>
            </section>
        }
    }
}

fn main() {
    wasm_logger::init(wasm_logger::Config::new(log::Level::Trace));
    console_error_panic_hook::set_once();
    yew::start_app::<LockWatch>();
}
